﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { RouteProp } from "@react-navigation/native";
import { booleanPointInPolygon, circle } from '@turf/turf';
import * as Notifications from 'expo-notifications';
import { observer } from "mobx-react";
import React, { useEffect } from "react";
import { AppState, Image, Keyboard, Platform, TouchableOpacity, TouchableWithoutFeedback } from "react-native";
import MapView, { LatLng, Marker, Polygon } from "react-native-maps";
import { PERMISSIONS } from "react-native-permissions";
import Spokestack from 'react-native-spokestack';
import { landmarkStore } from "../../../data/landmarks/store";
import { Landmark } from "../../../data/types";
import { AuthTabsNavigationProp, AuthTabsParamList as AuthTabsParamList } from "../../../navigation/AuthorizedNavigator";
import { MapStackNavigationProp, MapStackParamList } from "../../../navigation/MapNavigator";
import { NotifType } from "../../../types";
import { checkVoicePermissions, getMapPermissions, lmTypes } from "../../../utils/GlobalUtils";
import Badge from "../../Badge";
import { IconButton } from "../../Buttons";
import NearbyLandmarksPanel from "../Panels/NearbyLandmarksPanel";
import { VoicePanel } from "../Panels/VoicePanel";
import mapStyles from "./Map.styles";
import { useOutdoorMapState } from "./useMapState";

/**
 * An interface representing the user location retrieved from [expo-location]{@link https://docs.expo.dev/versions/latest/sdk/location/}.
 * @category Map
 */
export interface UserLocation {
    latitude: number;
    longitude: number;
    heading?: number;
}

export type MapStackRouteProp = RouteProp<MapStackParamList, 'Outdoor'>;

export type AuthTabsMapRouteProp = RouteProp<AuthTabsParamList, 'Map'>;

/**
 * The screen component containing the Map and all related functionality. Uses [react-native-maps]{@link https://github.com/react-native-maps/react-native-maps}
 * @category Map
 * @component
 */

interface OutdoorMapProps {
    mapNavigation: MapStackNavigationProp, 
    authNavigation: AuthTabsNavigationProp, 
    route: AuthTabsMapRouteProp,
    focusLandmark: (landmark: Landmark) => void,
    toggleLmDetails: (state: boolean) => void,
    toggleLmAdd: (state: boolean) => void,
    landmarks: Landmark[]
    applyFilters: (landmarks: Landmark[]) => Landmark[]
    promptAddLandmark: (longitude: number, latitude: number) => void
}

const OutdoorMap: React.FC<OutdoorMapProps> = (props) => {
    const mapState = useOutdoorMapState()

    /**
     * If the ReactNavigation route prop changes, check if it contains incoming selected landmarks, display them if there are. This will be triggered by incoming notifcations 
     * (See the AuthorizedNavigator page for the useEffect that will trigger this)
     */
     useEffect(() => {
        if (props.route?.params?.selectedLandmark) {
            landmarkStore.setSelectedLandmarkId(props.route?.params?.selectedLandmark)
        }
        if (props.route?.params?.selectedLandmarks) {
            mapState.toggleNearbyLmPanel(true)
        }
    }, [props.route])

    /**
     * Toggle the lm details panel when a new selected landmark is detected (triggered by pressing on a map marker, or from the list of nearby landmarks)
     */
     useEffect(() => {
        console.log("[LandmarkDetails]: Landmark selected - " + landmarkStore.selectedLandmarkId)
        if (landmarkStore.selectedLandmarkId) {
            const landmark = props.landmarks.find(lm => lm.id == landmarkStore.selectedLandmarkId)
            mapState.mapRef.current.animateToRegion({latitude: landmark.latitude, longitude: landmark.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01})
            props.toggleLmDetails(true)
        }
    }, [landmarkStore.selectedLandmarkId])

    /**
     * Move to pressed location when newlandmark changes
     */
     useEffect(() => {
        if (landmarkStore.selectedLandmarkId) {
            mapState.mapRef.current.animateToRegion({latitude: landmarkStore.newLandmark.latitude, longitude: landmarkStore.newLandmark.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01})
        }
    }, [landmarkStore.newLandmark])

    /**
     * Gets speech permissions from user, runs every time app is brought to foreground
     */
     useEffect(() => {
        const getSpeechPermissions = async () => {
            if (AppState.currentState == 'active') {
                await getMapPermissions()
                console.log('[Permissions]: Checking voice permissions...')
                if (Platform.OS == 'android') {        
                    const permitted = await checkVoicePermissions([PERMISSIONS.ANDROID.RECORD_AUDIO])
                    mapState.toggleVoicePermission(permitted)
                    if (permitted) console.log('[Permissions]: Voice permission granted')
                    else console.log('[Permissions]: Voice permission denied')
                }
                else if (Platform.OS == 'ios') {
                    const permitted = await checkVoicePermissions([PERMISSIONS.IOS.SPEECH_RECOGNITION, PERMISSIONS.IOS.MICROPHONE])
                    mapState.toggleVoicePermission(permitted)
                    if (permitted) console.log('[Permissions]: Voice permission granted')
                    else console.log('[Permissions]: Voice permission denied')
                }
            }   
        }
        getSpeechPermissions()
        }, [AppState.currentState])
    
        /**
         * Gets foreground location permissions from user, runs every time app is brought to foreground
         */
         useEffect(() => {
            const checkForegroundLocationPermissions = async () =>  {
                if (AppState.currentState == 'active') {
                    console.log('Checking location permissions...')
                    if (Platform.OS == 'android') {        
                        const permitted = await checkVoicePermissions([PERMISSIONS.ANDROID.ACCESS_COARSE_LOCATION, PERMISSIONS.ANDROID.ACCESS_FINE_LOCATION])
                        mapState.toggleFgroundLocationPermission(permitted)
                        if (permitted) console.log('[Permissions]: Location permission granted')
                        else console.log('[Permissions]: Location permission denied')
                    }
                    else if (Platform.OS == 'ios') {
                        const permitted = await checkVoicePermissions([PERMISSIONS.IOS.LOCATION_WHEN_IN_USE])
                        mapState.toggleFgroundLocationPermission(permitted)
                        if (permitted) console.log('[Permissions]: Location permission granted')
                        else console.log('[Permissions]: Location permission denied')
                    }
                }
            }
            checkForegroundLocationPermissions();
        }, [AppState.currentState])
    
        /**
         * Gets background location permissions from user, runs every time app is brought to foreground
         */
        useEffect(() => {
            const checkBackgroundLocationPermissions = async () =>  {
                if (AppState.currentState == 'active') {
                    if (Platform.OS == 'android') {        
                        const permitted = await checkVoicePermissions([PERMISSIONS.ANDROID.ACCESS_BACKGROUND_LOCATION])
                        mapState.toggleBgroundLocationPermission(permitted)
                        if (permitted) console.log('[Permissions]: Background location permission granted')
                        else console.log('[Permissions]: Background location permission denied')
                    }
                    else if (Platform.OS == 'ios') {
                        const permitted = await checkVoicePermissions([PERMISSIONS.IOS.LOCATION_ALWAYS])
                        mapState.toggleBgroundLocationPermission(permitted)
                        if (permitted) console.log('[Permissions]: Background location permission granted')
                        else console.log('[Permissions]: Background location permission denied')
                    }
                }
            }
            checkBackgroundLocationPermissions();
        }, [AppState.currentState])
    
        /**
         * Gets net location permission the existing location permission states. It will check foreground and background permissions and AppState, 
         * then from that it will decide if location-enabled features should be activated (through the mapState state values).
         */
         useEffect(() => {
            const updateLocationPermissionOnAppStateChange = async () => {
                const netLocationPermissions = mapState.bgroundLocationPermission || (mapState.fgroundLocationPermission && AppState.currentState == 'active')
                console.log('[Permissions]: Appstate, or location permissions changed, net location permissions found to be: ' + netLocationPermissions)
                mapState.toggleLocationPermitted(netLocationPermissions)
            }
            updateLocationPermissionOnAppStateChange()
        }, [AppState.currentState, mapState.bgroundLocationPermission, mapState.fgroundLocationPermission]) 


    /**
     * Animates the map to fly over to and focus on the user's location.
     */
     const flyToUser = () => {
        console.log('[Map]: Centering on user')
        if (mapState.userLocation) {
            mapState.mapRef.current?.animateToRegion({latitude: mapState.userLocation.latitude, longitude: mapState.userLocation.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01})
        }
    }

    /**
     * Activates speech recognition and opens the voice panel
     */
     const startSpeech = () => {
        props.toggleLmDetails(false);
        props.toggleLmAdd(false);
        Spokestack.activate()
    }

    /**
     * Gets initial region that map should zoom into from current user location
     */
    const getInitialRegion = () => {
        if (mapState.userLocation) {
            return {latitude: mapState.userLocation.latitude, longitude: mapState.userLocation.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01}
        }
        return undefined
    }

    /**
     * Method that runs every time user location changes, updates user location state in memory and checks if any landmarks are nearby
     */
    const updateLocation = async (coord: LatLng) => {
        mapState.setUserLocation(coord)
        // get 10m radius around user
        const userAlertRadius = circle([coord.longitude, coord.latitude], 10, {units: 'meters'})

        // check each landmark to see if its inside user radius. if it is, and it isn't already in the list of notified landmarks, add it
        const newLandmarksNearUser = props.landmarks?.filter(lm => {
            const landmarkNearUser = booleanPointInPolygon([lm.longitude, lm.latitude], userAlertRadius)
            return landmarkNearUser
        })

        // to prevent duplicate notifications make a list of landmarks that weren't previously near the user. 
        // these are the only ones that the user will be notified of
        const newLandmarksNotPreviouslyNearUser = newLandmarksNearUser.filter(lm => mapState.landmarksNearUser.some(origLm => lm == origLm.id))

        // update list
        mapState.setLandmarksNearUser(newLandmarksNearUser)

        // if there are any new landmarks near user, create a notification for them and send it
        if (newLandmarksNotPreviouslyNearUser?.length > 0) {
            const body = newLandmarksNotPreviouslyNearUser.length > 1 ? "There are new landmarks near by. Tap here to view" : "There is a new landmark close by. Tap here to view"
            const notifType: NotifType = newLandmarksNotPreviouslyNearUser.length > 1 ? 'near-landmarks' : 'near-landmark'
            const data = {notif_type: notifType, landmarks: newLandmarksNotPreviouslyNearUser.length == 1 ? newLandmarksNearUser : null}
            await Notifications.scheduleNotificationAsync({
                content: {
                  title: "⚠ Landmarks close by ⚠",
                  body: body,
                  data: data
                },
                trigger: { seconds: 2 },
              });
        }
    }

    const focusNearbyLandmarks = () => {
        if (mapState.landmarksNearUser?.length > 1) {
            mapState.toggleNearbyLmPanel(true)
        }
        else if (mapState.landmarksNearUser?.length === 1) { 
            landmarkStore.setSelectedLandmarkId(mapState.landmarksNearUser[0].id)
        }
    }

    return (
        <TouchableWithoutFeedback>
            <>
            {/*Main map component*/}
            <MapView 
                toolbarEnabled={false}
                onPress={() => Keyboard.dismiss()}
                testID="mapView"
                ref={mapState.mapRef} 
                style={{width: '100%', height: '100%'}}
                initialRegion={getInitialRegion()} 
                onLongPress={(e) => props.promptAddLandmark(e.nativeEvent.coordinate.longitude, e.nativeEvent.coordinate.latitude)} 
                showsUserLocation={mapState.locationPermitted} 
                onUserLocationChange={e => updateLocation(e.nativeEvent.coordinate)}
                followsUserLocation={mapState.followUser}
                showsMyLocationButton={false}>
            <Polygon // polygon for cameron library
              coordinates={[
                {latitude: 53.527190, longitude: -113.524205 },
                {latitude: 53.526510, longitude: -113.524205 },
                {latitude: 53.526510, longitude: -113.523452 },
                {latitude: 53.527190, longitude: -113.523452 },
                // { name: "5", latitude: 60, longitude: -105 },
              ]}
              fillColor={`rgba(100,100,200,0.3)`}
              strokeWidth={2.5}
              tappable={true}
              onPress={() => props.mapNavigation.navigate("Indoor")}
              />
            {props.applyFilters(props.landmarks)?.map((landmark) => {
                let trackChanges = false;
                if (landmark?.id == landmarkStore.selectedLandmarkId) {
                    trackChanges = true;
                }
                return (
                    <Marker 
                        tracksViewChanges={trackChanges}
                        onPress={() => props.focusLandmark(landmark)}
                        key={landmark.id} 
                        coordinate={{latitude: landmark.latitude as number, longitude: landmark.longitude as number}} >
                        { landmark.landmark_type ? <Image style={{height: 35, width: 25}} source={lmTypes[landmark.landmark_type].image} /> : null}
                    </Marker>)})}
            </MapView>
            
            {/*Map buttons*/}
            {mapState.landmarksNearUser?.length > 0 ? 
            <TouchableOpacity style={[mapStyles.lowerMapButton, mapStyles.alertButton]} onPress={focusNearbyLandmarks}>
                <FontAwesome name='exclamation-triangle' size={20} color='white' />
                <Badge positioning={{bottom: 7, right: 4}} value={mapState.landmarksNearUser.length}/>
            </TouchableOpacity> : null}
            {mapState.locationPermitted && mapState.voicePermission ? 
            <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.voiceButton]} icon="microphone" onPress={startSpeech}/>: null}
            <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.addLandmarkButton]} icon="plus" onPress={() => props.promptAddLandmark(mapState.userLocation.longitude, mapState.userLocation.latitude)}/>
            <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.userLocationButton]} icon="location-arrow" onPress={flyToUser}/>
            <NearbyLandmarksPanel 
                focusLandmark={props.focusLandmark}
                nearbyLmPanelVisible={mapState.nearbyLmPanelVisible}
                toggleAlertedLmPanel={mapState.toggleNearbyLmPanel}
                nearbyLandmarks={mapState.landmarksNearUser}/>
            {/*Map Panels*/}
            {mapState.voicePermission && mapState.locationPermitted ? 
            <VoicePanel 
                landmarksNearby={mapState.landmarksNearUser.length > 0} 
                toggleAlertedLandmarksVisible={mapState.toggleNearbyLmPanel}
                navigation={props.authNavigation}
                userCoords={{longitude: mapState.userLocation?.longitude, latitude: mapState.userLocation?.latitude}}
                toggleVoiceVisible={mapState.toggleVoiceVisible} 
                toggleLmDetails={props.toggleLmDetails}
                voiceVisible={mapState.voiceVisible}
                /> : null }
            </>
        </TouchableWithoutFeedback> )
}

export default observer(OutdoorMap);