﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { observer } from "mobx-react-lite";
import React, { MutableRefObject, useEffect, useRef, useState } from "react";
import { FlatList, Keyboard, ListRenderItem, StyleSheet, Text, TextInput, View } from "react-native";
import { useAddComment, useDeleteComment, useLandmarkComments, useUpdateComment } from "../../../../data/comments/hooks";
import { landmarkStore } from "../../../../data/landmarks/store";
import { useProfile } from "../../../../data/profile/hooks";
import { LMComment } from "../../../../data/types";
import { authStore } from "../../../../libs/auth/AuthStore";
import { CommentView } from "./CommentView";

interface CommentsContainerProps {
    setKeyboardOpened: (state: boolean) => void
    keyboardOpened: boolean
}

/**
 * Renders all [comments]{@link LMComment} associated with the {@linkcode selectedLandmark} as items for the [FlatList]{@link https://reactnative.dev/docs/flatlist} in this component.
*/
const CommentsContainer: React.FC<CommentsContainerProps> = (props) => {
    /**
     * Holds text of the current {@link LMComment} being added to this {@link Landmark}.
     */
    const [newCommentId, setNewComment] = useState<string>("");
    /**
     * Holds id of the focused {@link LMComment} in the comment list.
     */
    const [focusedCommentId, setFocusedComment] = useState<string>("");
    /**
     * Holds state of the {@link LMComment} being currently edited.
     */
    const [commentBeingEdited, setCommentBeingEdited] = useState<LMComment | undefined>(undefined);

    /**
     * Holds a reference to the Flatlist containing the comments.
     */
    const commentListRef = useRef<FlatList>();
     /**
      * Holds a reference to the text input for posting a new comment
      */
    const commentTextInputRef = useRef<TextInput>();
    const comments = useLandmarkComments()
    const addComment = useAddComment()
    const updateComment = useUpdateComment() 
    const deleteComment = useDeleteComment() 

    const profile = useProfile()

    useEffect(() => {
        /**
         * Resets the {@linkcode updateComment} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode updateCommentStatus} value from the {@link useComments} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateComment = () => {
            if (updateComment.isSuccess) {
                commentListRef.current?.scrollToItem({animated: true, item: commentBeingEdited});
                setCommentBeingEdited(undefined)
                Keyboard.dismiss();
            }
        }
        resetUpdateComment();
    }, [updateComment.isSuccess]);

    useEffect(() => {
        /**
         * Resets the {@link addCommentAsync} mutation on successful add.
         * Embedded in a useEffect that listens to the {@link addCommentAsync} value from the {@link useComment} hook.
         * @memberOf LandmarkDetailsLandmark
         */
        const resetAddCommentOnSuccess = () => {
            if (addComment.isSuccess) {
                addComment.reset();
            }
            setNewComment('')
            commentListRef.current?.scrollToIndex({animated: true, index: 0});
            Keyboard.dismiss();
        }
        resetAddCommentOnSuccess();
    }, [addComment.isSuccess]);

    useEffect(() => {
        /**
         * Clears selected comment when {@linkcode newCommentId} state changes.
         * @memberOf LandmarkDetails
         */
        const clearSelectedOnNewCommentChange = () => {
            setFocusedComment('')
        }
        clearSelectedOnNewCommentChange();
    }, [newCommentId]);

    /**
     * Set a comment to be focused in the comment box. It will expand and show delete and edit controls for that comment. 
     * If the comment id given is already selected, the comment will be unselected
     */
     const focusComment = (id: string) => {
        if (focusedCommentId == id) {
            setFocusedComment("");
        }
        else {
            setFocusedComment(id);
        }
    }

    /**
     * Set a comment to be the comment currently edited. 
     */
     const startEditingComment = (comment: LMComment) => {
        setCommentBeingEdited(comment);
        commentTextInputRef.current.focus();
    }

    /**
     * Flatlist render item method for each comment. 
     * @param 
     * @returns 
     */
    const renderComment: ListRenderItem<LMComment> = ({item, index}) => {
        const selected = item.id == focusedCommentId;
        const belowSelected = comments[index - 1]?.id == focusedCommentId;
        const latestComment = index == 0;
        return (
            <View>
                {/* Some conditional rendering to make the borders look nice */}
                {latestComment ? null : <View style={[selected || belowSelected ? {marginVertical: 0} : {marginHorizontal: 10}, {height: 1, borderBottomWidth: 1, borderColor: 'lightgray'}]}></View> }
                <CommentView comment={item} selected={selected} focusComment={focusComment} startEditingComment={startEditingComment} deleteComment={async () => await deleteComment.mutateAsync(item.id)} />
            </View>
        );
    };

    /**
     * || DEPRECATED ||
     * Sets a flag that tracks whether keyboard is shown
     */
    // React.useEffect(() => {
    //     if (Platform.OS == 'android') {
    //         Keyboard.addListener('keyboardDidShow', () => props.setKeyboardOpened(true))
    //         Keyboard.addListener('keyboardDidHide', clearStateOnKeybordDismiss);
    //     }
    
    //     // cleanup function
    //     return () => {
    //       Keyboard.removeAllListeners('keyboardDidHide');
    //       Keyboard.removeAllListeners('keyboardDidShow');
    //     };
    //   }, []);

    /**
     * || DEPRECATED ||
     * Clears out the comment input if keyboard is dismissed without posting the comment **DEPRECATED**
     */
    // const clearStateOnKeybordDismiss = () => {
    //     props.setKeyboardOpened(false)
    //     props.setCommentBeingEdited(undefined);
    //     if (props.commentBeingEdited) {
    //         props.setCommentBeingEdited(undefined);
    //     }
    //     if (props.newCommentId) {
    //         props.setNewComment('');
    //     }
    // }

    /**
     * Simple check to see if the landmark has any comments
     */
    const hasComments = () => {
        return comments.data?.length > 0
    }

    const tryAddComment = async () => {
        await addComment.mutateAsync({
            edited: false,
            content: newCommentId,
            landmark: landmarkStore?.selectedLandmarkId,
            poster: authStore?.userId,
            poster_name: profile?.data?.username,
            id: ''
        })
    }

    return (  
    <View style={[styles.commentContainer, {borderBottomWidth: 1, borderColor: 'lightgray', paddingBottom: 20, marginBottom: 10}]}>
        {hasComments() ? 
        <>
            <Text style={{color: 'white', marginBottom: 20}}>Comments: </Text>
            <FlatList<LMComment>
                nestedScrollEnabled={true}
                ref={commentListRef}
                keyExtractor={i => i.id}
                data={comments.data} 
                extraData={focusedCommentId}
                renderItem={renderComment}
                style={{backgroundColor: 'white'}}
                getItemLayout={(data, index) => ({length: comments.data.length, offset: comments.data.length * index, index})}/>
        </> : 
        <Text style={{marginVertical: 20, color: 'white'}}>Be the first to comment on this landmark</Text> }
        <View style={{height: 1, borderBottomWidth: 1, borderColor: 'lightgray'}}></View>
        <View style={{flexDirection: 'row', backgroundColor: 'white', paddingRight: 15}}>
            <TextInput 
                ref={commentTextInputRef}
                placeholder="Add comment..." 
                onChangeText={commentBeingEdited ? value => setCommentBeingEdited({...commentBeingEdited, content: value}) : value => setNewComment(value)} 
                value={commentBeingEdited ? commentBeingEdited.content : newCommentId}
                placeholderTextColor="gray" 
                style={{padding: 10, flex: 5}}
                multiline={true} />
            {newCommentId || commentBeingEdited ? 
            <View style={{alignItems: 'center', justifyContent: 'space-between', flexDirection: 'row', flex: 1}}>
                <FontAwesome name="times" size={20} color="lightgray" style={{ backgroundColor: 'white',  }} onPress={Keyboard.dismiss} />
                {commentBeingEdited ?
                <FontAwesome name="check" size={20} style={{ marginLeft: 15, marginRight: 15, backgroundColor: 'white', padding: 'auto'}} onPress={async () => await updateComment.mutateAsync(commentBeingEdited)} /> :
                <FontAwesome 
                    name="paper-plane" 
                    size={20} 
                    style={{ backgroundColor: 'white', padding: 'auto'}} 
                    onPress={async () => await tryAddComment()} /> }
            </View> : null}
        </View>
    </View>)
}

const styles = StyleSheet.create({
    input: {
        padding: 5,
        color: 'black'
    },
    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
})

export default observer(CommentsContainer)