﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { observer } from "mobx-react-lite";
import React, { MutableRefObject, useEffect } from "react";
import { FlatList, Image, ScrollView, StyleSheet, Text, TextInput, View } from "react-native";
import Picker from "react-native-picker-select";
import { landmarkStore } from "../../../../data/landmarks/store";
import { Landmark, LMComment } from "../../../../data/types";
import { lmTypes } from "../../../../utils/GlobalUtils";
import { Separator } from "../../../Separator";
import CommentsContainer from "./CommentsContainer";
import { LandmarkPhotos } from "./LandmarkPhotos";

interface DetailsBodyProps {
    editingEnabled: boolean,
    landmark?: Landmark,
    setSelectedImage: (index: number) => void 
    tryDeletePhoto: (photoId: string) => void
    toggleLmDetails: (state: boolean) => void
    setKeyboardOpened: (state: boolean) => void
    keyboardOpened: boolean
    profileId: string
    processingPhoto: boolean
    setProcessingPhoto: (state: boolean) => void
}

/**
 * Component that renders the body of the Landmark details panel
*/
const DetailsBody: React.FC<DetailsBodyProps> = (props) => {

    useEffect(() => {
        if (props.editingEnabled) {
            console.log("[LandmarkDetails]: Editing is enabled")
            landmarkStore.setUpdatedLandmark(props.landmark)
        }
        else {
            landmarkStore.setUpdatedLandmark(undefined)
        }
    }, [props.editingEnabled])
    
    /**
     * Sub-component that renders picker for landmark types
     * @param 
     */
    const LandmarkTypePicker: React.FC = () => {
        return (
        <View style={{flexDirection: 'row', marginBottom: 20, justifyContent: "space-between"}}> 
            {landmarkStore.updatedLandmark?.landmark_type ? 
            <>
                <Picker
                    style={{
                        inputIOS: {color: 'white'}, 
                        inputAndroid: {color: 'white'},
                        iconContainer: {flex: 1, justifyContent: 'center', height: '100%'},
                        viewContainer: {padding: 5, elevation: 1, flex: 1, justifyContent: 'center'}, placeholder: {color: 'white'}}}
                    placeholder={{}}
                    value={landmarkStore.updatedLandmark?.landmark_type}
                    onValueChange={(value) => {
                        landmarkStore.setUpdatedLandmark({...landmarkStore.updatedLandmark, landmark_type: value, title: lmTypes[value].label})
                    }}
                    useNativeAndroidPickerStyle={true}
                    items={Object.keys(lmTypes)?.filter(icon => parseInt(icon) != props.landmark?.landmark_type).map(icon => {
                        return (
                            {label: lmTypes[parseInt(icon)].label.toUpperCase(), value: icon, key: icon}
                        )})} />
                {landmarkStore.updatedLandmark ? <Image style={{marginLeft: 20}} source={lmTypes[landmarkStore.updatedLandmark?.landmark_type].image}/> : null}
            </>
            : null}
        </View>
        )
    }

    /**
     * Sub-component that renders the view to be displayed when editing is disabled
     * @param 
     */
    const EditingDisabledUpperView: React.FC = () => {
        return (
            <View style={{flexDirection: 'row', justifyContent: 'space-between'}}>
                <View style={{flex: 8, flexDirection: 'column', marginBottom: 20}}>
                    <Text style={{color: 'white', marginBottom: 10, fontSize: 15}}>{lmTypes[props.landmark?.landmark_type]?.label.toUpperCase()}</Text>
                    <ScrollView nestedScrollEnabled={true}>
                        <Text style={{color: 'white', fontSize: 13}}>{props.landmark?.description}</Text>
                    </ScrollView>
                </View>
                {props.landmark?.landmark_type ? <Image source={lmTypes[props.landmark?.landmark_type].image} /> : null}
            </View>
        )
    }

    return (
        <ScrollView nestedScrollEnabled={true} contentContainerStyle={{justifyContent: 'space-between'}} style={{flex: 1, marginHorizontal: 20}}>
            {props.editingEnabled ?
            <>
                <LandmarkTypePicker />
                <Separator style={{marginBottom: 20, opacity: .5}} color="lightgray" />
                <Text style={{color: 'white', marginBottom: 10}}>Description</Text>
                <ScrollView nestedScrollEnabled={true} style={{backgroundColor: 'white', marginBottom: 20}}>
                    <TextInput 
                        multiline={true} 
                        style={[styles.input, {fontSize: 13, marginBottom: 10}]}
                        onChangeText={text => landmarkStore.setUpdatedLandmark({...landmarkStore.updatedLandmark, description: text})} 
                        value={landmarkStore.updatedLandmark?.description}/>
                </ScrollView>
            </>: <EditingDisabledUpperView />}
            {!props.editingEnabled ?
            <CommentsContainer
                setKeyboardOpened={props.setKeyboardOpened}
                keyboardOpened={props.keyboardOpened}/> : null}
            {!props.editingEnabled ?
            <LandmarkPhotos 
                profileId={props.profileId}
                tryDeletePhoto={props.tryDeletePhoto}
                toggleLmDetails={props.toggleLmDetails}
                editingEnabled={props.editingEnabled}
                landmark={props.landmark}
                setSelectedImage={props.setSelectedImage}
                processingPhoto={props.processingPhoto}
                setProcessingPhoto={props.setProcessingPhoto}/>
             : null}
            
        </ScrollView>
    )
}

const styles = StyleSheet.create({
    input: {
        padding: 5,
        color: 'black'
    },
    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
})

export default observer(DetailsBody)