﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { observer } from "mobx-react";
import React, { useEffect } from "react";
import { View, Text, TouchableOpacity, StyleSheet } from "react-native";
import { QueryStatus } from "react-query";
import { useAddLandmarkPhoto, useCheckLandmarkRateUserPairing, useDeleteLandmark, useDeleteLandmarkPhoto, useRateLandmark } from "../../../../data/landmarks/hooks";
import { landmarkStore } from "../../../../data/landmarks/store";
import { Landmark, UserProfile } from "../../../../data/types";
import { authStore } from "../../../../libs/auth/AuthStore";

interface DetailsHeaderProps {
    landmark?: Landmark,
    editingEnabled: boolean,
    toggleEditing: (state: boolean) => void,
    editLandmark: () => void,
    toggleDetailsPanel: (state: boolean) => void,
    profile?: UserProfile,
    processingPhoto: boolean,
}

/**
 * Component that renders the landmark details panel header
 * @param 
 */
const DetailsHeader: React.FC<DetailsHeaderProps> = (props) => {
    const {isLoading: addPhotoLoading} = useAddLandmarkPhoto()
    const {isLoading: deletePhotoLoading} = useDeleteLandmarkPhoto()
    const {mutateAsync: deleteLandmark} = useDeleteLandmark() 
    const checkIfLandmarkRatedByUser = useCheckLandmarkRateUserPairing()
    const rateLandmark = useRateLandmark()

    const photosAreBusy = () => {
        return props.processingPhoto || addPhotoLoading || deletePhotoLoading
    }

    useEffect(() => {
        /**
         * Refetch the user's rate pairing for the current landmark when {@linkcode landmark} state changes.
         * @memberOf LandmarkDetails
         */
        const refetechRateStatus = async () => {
            await checkIfLandmarkRatedByUser.refetch();
        }
        refetechRateStatus();
    }, [props.landmark]);

    useEffect(() => {},
    [props.landmark, rateLandmark.status])

    const HeaderContent: React.FC = () => {
        // landmark is owned by user
        if (authStore.userId == props.landmark?.user) {
            // editing is enabled
            if (props.editingEnabled) { 
                return (
                    <View style={{flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%'}}>
                        <FontAwesome color="white" size={25} name="close" onPress={() => props.toggleEditing(false)}/>
                        {landmarkStore.updatedLandmark?.description && landmarkStore.updatedLandmark.landmark_type ? // only display check if fields are populated
                        <FontAwesome style={{marginTop: 2}} size={25} color="white" name="check" onPress={async () => props.editLandmark()}/> : null}
                    </View>
                )
            }
            // editing is disabled
            else { 
                return (
                    
                    <View style={{flexDirection: 'row', alignItems: "center", justifyContent: 'space-between', width: '100%'}}>
                        <>  
                            <View style={{flexDirection: 'row', alignItems: "center"}}>
                                <FontAwesome style={{marginRight: 20, marginTop: 2}} size={25} color="white" name="edit" onPress={() => props.toggleEditing(true)}/> 
                                <FontAwesome style={{marginBottom: 2}} color="white" size={25} name="trash" onPress={async () => deleteLandmark(landmarkStore.selectedLandmarkId)}/>
                                <Text style={{color: 'white', fontSize: 20, marginTop: 2, marginLeft: 40}} >{props.landmark?.rating}</Text> 
                                <FontAwesome style={{marginLeft: 5, marginBottom: 2, marginRight: 10}} color="white" size={25} name="thumbs-up"/>
                            </View>
                            <FontAwesome color="white" size={25} style={{ }} name="close" onPress={() => props.toggleDetailsPanel(false)}/> 
                        </>
                    </View>
                )
            }
        }
        // landmark is not owned by user
        else {
            return (
                <View style={{flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%'}}>
                    {checkIfLandmarkRatedByUser.data ?  // landmark has already been liked by the current user
                    <View style={{flexDirection: 'row'}} > 
                        <Text style={{color: 'white', fontSize: 20, marginTop: 2}} >{props.landmark.rating}</Text> 
                        <FontAwesome style={{marginLeft: 5, marginTop: 2, marginRight: 30}} color="white" size={25} name="thumbs-up"/>
                        <Text style={{color: 'lightgray', fontSize: 20, marginTop: 2, opacity: 0.7}} >You liked this </Text>
                        <TouchableOpacity style={{flexDirection: 'row'}} onPress={async () => rateLandmark.mutateAsync(-1)}>
                            <Text style={{color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8}} >(</Text>
                            <Text style={{color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8, textDecorationLine: "underline"}} >Undo</Text>
                            <Text style={{color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8}} >)</Text>
                        </TouchableOpacity>
                    </View> : // landmark has not been liked by user
                    <TouchableOpacity onPress={async () => { //this touchable will add a like to this landmark
                            if (authStore.userId !== props.landmark?.id) {
                                await rateLandmark.mutateAsync(1);
                            }
                        }}>
                        <View style={{flexDirection: 'row'}} >
                            <Text style={{color: 'white', fontSize: 20, marginTop: 2}} >{props.landmark?.rating}</Text>
                            <FontAwesome style={{marginLeft: 5, marginTop: 2}} color="white" size={25} name="thumbs-up"/>
                        </View>
                    </TouchableOpacity>}
                    <FontAwesome color="white" size={25} name="close" onPress={() => props.toggleDetailsPanel(false)}/> 
                </View>
            )
        }
    }

    return (
        <View style={styles.detailsHeader}>
        {!photosAreBusy() ?
            <HeaderContent />
        :null}
        </View>
    )
}

const styles = StyleSheet.create({
    detailsHeader: {
        borderBottomWidth: 1,
        borderColor: 'white',
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center',
        backgroundColor: '#df3f3f',
        marginBottom: 10,
        paddingVertical: 10,
        paddingHorizontal: 20
    },
})

export default observer(DetailsHeader);