﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { observer } from "mobx-react-lite";
import React, { memo, useEffect, useState } from "react";
import { ActivityIndicator, Alert, Dimensions, Image, Keyboard, Platform, SafeAreaView, StyleSheet, Text, TouchableOpacity, View } from "react-native";
import { ScrollView } from "react-native-gesture-handler";
import Modal from 'react-native-modal';
import { useAddComment, useDeleteComment, useUpdateComment } from "../../../../data/comments/hooks";
import { useDeleteLandmark, useDeleteLandmarkPhoto, useLandmarkDetails, useUpdateLandmark } from "../../../../data/landmarks/hooks";
import { landmarkStore } from "../../../../data/landmarks/store";
import { useProfile } from "../../../../data/profile/hooks";
import { authStore } from "../../../../libs/auth/AuthStore";
import { colors } from "../../../../utils/GlobalUtils";
import { IconButton, PrimaryButton } from "../../../Buttons";
import DetailsBody from "./DetailsBody";
import DetailsHeader from "./DetailsHeader";

/**
 * Props for the {@link LandmarkDetails} component.
 */
export interface LandmarkDetailsProps {
    /**
     * The {@link Landmark} object being displayed/edited in the {@link LandmarkDetails} modal. 
     */
    landmarkId: string | undefined
    /**
     * The state setter for {@link Landmark} object being displayed/edited in the {@link LandmarkDetails} modal. 
     */
    setLandmark: (landmark: string) => void; 
    /**
     * A callback passed from the parent {@link Map} that toggles the visibility of the {@link LandmarkDetails} modal.
     */
    toggleDetailsPanel: (state: boolean) => void;
    /**
     * A callback passed from the parent {@link Map} that toggles the ability to edit the {@link Landmark} in {@link LandmarkDetails} modal.
     */
    setEditing: (state: boolean) => void;
    /**
     * A flag that determines whether the properties of the {@link Landmark} displayed in the {@link LandmarkDetails} modal can be edited
     */
    editingEnabled: boolean,
    visible: boolean,
    toggleLmDetails: (state: boolean) => void
}

/**
 * Component that renders the details of a selected {@link Landmark} and allows the user to edit those details. Contained within a [react-native-modal]{@link https://github.com/react-native-modal/react-native-modal}.
 * @component
 * @category Map
 */
const LandmarkDetails: React.FC<LandmarkDetailsProps> = ({landmarkId, setLandmark, toggleDetailsPanel, setEditing, editingEnabled, visible, toggleLmDetails}) => {
    /**
     * Flag that tracks if the keyboard is open
     */
    const [keyboardOpened, setKeyboardOpened] = useState<boolean>(false);  
    
    /**
     * State that tracks the index of the currently selected photo, if any are selected
     */
    const [selectedImage, setSelectedImage] = useState<number>(-1)
    const [processingPhoto, setProcessingPhoto] = useState<boolean>(false)
    
    const landmarkDetails = useLandmarkDetails()
    const updateLandmark = useUpdateLandmark() 
    const deleteLandmark = useDeleteLandmark()
    const deleteLandmarkPhoto = useDeleteLandmarkPhoto()
    const {status: addCommentStatus} = useAddComment()
    const {status: updateCommentStatus} = useUpdateComment()
    const {status: deleteCommentStatus} = useDeleteComment()
    const profile = useProfile()

    useEffect(() => {
    const keyboardDidShowListener = Keyboard.addListener(
      'keyboardDidShow',
      () => {
        setKeyboardOpened(true); // or some other action
      }
    );
    const keyboardDidHideListener = Keyboard.addListener(
      'keyboardDidHide',
      () => {
        setKeyboardOpened(false); // or some other action
      }
    );

    return () => {
      keyboardDidHideListener.remove();
      keyboardDidShowListener.remove();
    };
  }, []);

    useEffect(() => {
        /**
         * Resets the {@linkcode rateLandmark} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode rateLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateLandmark = async () => {
            if (updateLandmark.isSuccess) {
                updateLandmark.reset();
                await landmarkDetails.refetch()
                setEditing(false);
            }
        }
        resetUpdateLandmark();
    }, [updateLandmark.status]);

    useEffect(() => {
        /**
         * Resets the {@linkcode deleteLandmark} mutation on successful delete.
         * Embedded in a useEffect that listens to the {@linkcode deleteLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
        const resetDeleteLMOnSuccess = () => {
            if (deleteLandmark.isSuccess) {
                deleteLandmark.reset();
                toggleDetailsPanel(false)
            }
        }
        resetDeleteLMOnSuccess();
    }, [deleteLandmark.status]);

    /**
     * Calls the {@linkcode updateLandmark} mutation from the {@link useLandmarks} hook and closes the modal once finished.
     */
    const editLandmark = async () => {
        if (landmarkStore.updatedLandmark) {
            await updateLandmark.mutateAsync();       
        }
        
        setEditing(false);
    }

    const getWindowWidth = () => {
        return Dimensions.get('window').width
    }

    /**
     * Prompts user for a confirmation to delete the photo that they just tried to delete
     */
    const tryDeletePhoto = (photoId: string) => {
        Alert.alert(
            'Confirm photo removal', 
            'Are you sure you want to delete this photo?', 
            [
                {text: 'Yes', 
                onPress: async () => {
                    await deleteLandmarkPhoto.mutateAsync(photoId)
                    setSelectedImage(-1)
                    await landmarkDetails.refetch()
                }},
                {text: 'No'} 
            ])
    }

    /**
     * Returns a height for the modal depending on if an image is maximzed, if the keyboard is opened, and if the current landmark has photos associated with it
     */
    const determineModalHeight = () => {
        if (selectedImage > -1) 
            return Dimensions.get("window").height 
        else if (keyboardOpened && Platform.OS == 'android' || editingEnabled || (profile?.data?.id != landmarkDetails?.data?.user && landmarkDetails?.data?.photos?.length == 0))
            return Dimensions.get("window").height * .5
        else if (landmarkDetails.data?.photos?.length > 0) 
            return Dimensions.get("window").height * .9 
        else
            return Dimensions.get("window").height * .7
    }

    return (
        <Modal 
            useNativeDriver={true}
            useNativeDriverForBackdrop={true}
            avoidKeyboard={false}
            onBackdropPress={() => {
                if (editingEnabled) {
                    Keyboard.dismiss();
                } else {
                    toggleLmDetails(false)   
                }
            }}
            style={{justifyContent: "flex-end", height: '100%', margin: 0}}
            isVisible={visible}>
            <SafeAreaView 
                style={[styles.container, {height: determineModalHeight()}]}>
                {selectedImage > -1 ?
                <View style={{ padding: 14}}>
                    <View style={{justifyContent: 'space-between', flexDirection: 'row'}}>
                        <TouchableOpacity onPress={() => setSelectedImage(-1)}>
                            <>
                                <Text style={{color: 'white', fontSize:20}}><FontAwesome name="arrow-left" color="white" size={20}/> Back</Text>
                            </>
                        </TouchableOpacity>
                        <IconButton style={{alignSelf: 'flex-end', marginBottom: 20}} icon="trash" color="white" size={25} onPress={() => tryDeletePhoto(landmarkDetails.data?.photos[selectedImage].id)}/>
                    </View>
                    <ScrollView style={{width: '100%', }}>
                        <Image style={{resizeMode: 'contain', alignSelf: 'center', height: Dimensions.get('window').height * .9, width: getWindowWidth()}} source={{uri: 'data:image/png;base64,' + landmarkDetails.data?.photos[selectedImage].image_b64}}/> 
                    </ScrollView>
                </View>
                :
                (landmarkDetails.isIdle || landmarkDetails.isSuccess) && 
                 updateLandmark.isIdle || updateLandmark.isSuccess && 
                 deleteLandmark.isIdle || deleteLandmark.isSuccess ?
                <>
                <DetailsHeader
                    processingPhoto={processingPhoto}
                    toggleDetailsPanel={toggleDetailsPanel}
                    landmark={landmarkDetails.data}
                    editLandmark={editLandmark}
                    editingEnabled={editingEnabled}
                    toggleEditing={setEditing}
                    profile={profile?.data} />
                <DetailsBody 
                    setProcessingPhoto={setProcessingPhoto}
                    processingPhoto={processingPhoto}
                    profileId={authStore.userId}
                    setKeyboardOpened={setKeyboardOpened}
                    keyboardOpened={keyboardOpened}
                    setSelectedImage={setSelectedImage}
                    landmark={landmarkDetails.data}
                    editingEnabled={editingEnabled}
                    tryDeletePhoto={tryDeletePhoto}
                    toggleLmDetails={toggleLmDetails}/>
                </> :
                <View style={{height: '100%', justifyContent: "center", alignItems: "center"}}>
                    {
                        deleteLandmark.isLoading || updateLandmark.isLoading || updateLandmark.isLoading || landmarkDetails.isLoading ? <ActivityIndicator color='white' size={40}/> :
                        deleteLandmark.isError || updateLandmark.isError || landmarkDetails.isError ? <PrimaryButton text="Okay" style={{borderColor: 'white', borderWidth: 1}} onPress={() => landmarkDetails.refetch()} /> : null
                    }
                    <Text style={{color: 'white', fontSize: 20, marginTop: 30}}>{
                        landmarkDetails.isLoading ? "Loading landmark..." : 
                        landmarkDetails.isError ? "Something went wrong trying to load the landmark" :
                        deleteLandmark.isLoading ? "Deleting landmark..." : 
                        deleteLandmark.isError ? "Something went wrong trying to delete the landmark" :
                        updateLandmark.isLoading ? "Updating landmark..." : 
                        updateLandmark.isError ? "Something went wrong trying to update the landmark" :
                        addCommentStatus == 'loading' ? "Adding comment..." : 
                        addCommentStatus == 'error' ? "Something went wrong trying to add the comment" : 
                        updateCommentStatus == 'loading' ? "Updating comment..." : 
                        updateCommentStatus == 'error' ? "Something went wrong trying to update the comment" :  
                        deleteCommentStatus == 'loading' ? "Deleting comment..." : 
                        deleteCommentStatus == 'error' ? "Something went wrong trying to delete the comment" : null} 
                    </Text>
                </View>}
            </SafeAreaView>
        </Modal>
    )
}

const styles = StyleSheet.create({
    container: {
        backgroundColor: colors.red,
    },
    detailsContainer: {
        flex: 1,
        marginHorizontal: 20
    },

    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
    title: {

    },
    input: {
        padding: 5,
        color: 'black'
    }
})

export default observer(LandmarkDetails);