import { FontAwesome } from "@expo/vector-icons"
import { ImageInfo } from "expo-image-picker/build/ImagePicker.types"
import React, { useEffect, useState } from "react"
import { ActivityIndicator, Image, ScrollView, Text, TouchableOpacity, View } from "react-native"
import { QueryStatus, UseMutationResult } from "react-query"
import { useAddLandmarkPhoto, useDeleteLandmarkPhoto } from "../../../../data/landmarks/hooks"
import { Landmark, LMPhoto } from "../../../../data/types"
import { IconButton, PrimaryButton } from "../../../Buttons"
import { PhotoPicker } from "../../../PhotoPicker"

interface LandmarkPhotosProps {
    toggleLmDetails: (state: boolean) => void
    landmark?: Landmark
    editingEnabled: boolean
    setSelectedImage: (index: number) => void
    tryDeletePhoto: (photoId: string) => void
    processingPhoto: boolean
    setProcessingPhoto: (state: boolean) => void
    profileId: string
}

export const LandmarkPhotos: React.FC<LandmarkPhotosProps> = (props) => {
    /**
     * Flag that toggles the photo source menu being displayed
    */
    const photoSourceMenuOpenedState = false
    const [photoSourceMenuOpened, togglePhotoSourceMenu] = useState<boolean>(photoSourceMenuOpenedState) 
    const [photoStatusMessage, setPhotoStatusMessage] = useState<string>()
    const [photoError, setPhotoError] = useState<boolean>(false)
    const addPhoto = useAddLandmarkPhoto()
    const deletePhoto = useDeleteLandmarkPhoto()

    /**
     * Handles selection of a selected photo. Adds the photo data to the database.
    */
     const _onPhotoSelected = async (result: ImageInfo) => {
        togglePhotoSourceMenu(false)
        const photo: LMPhoto = {id: '', image_b64: 'data:image/png;base64,' + result.base64, height: result.height, width: result.width, landmark: props.landmark?.id}
        props.setProcessingPhoto(false)
        await addPhoto.mutateAsync({landmarkId: props.landmark?.id, photo: photo})
    }

    useEffect(() => {
        if (deletePhoto.isError) {
            setPhotoStatusMessage('Error deleting photo')
            setPhotoError(true)
        }

        if (addPhoto.isError) {
            setPhotoStatusMessage('Error adding photo')
            setPhotoError(true)
        }
        
        if (addPhoto.isLoading) 
            setPhotoStatusMessage('Adding photo...')
        
        if (deletePhoto.isLoading) 
            setPhotoStatusMessage('Deleting photo...')

        if (deletePhoto.isSuccess || addPhoto.isSuccess) 
            setPhotoStatusMessage("")

        if (props.processingPhoto) 
            setPhotoStatusMessage("Processing photo...")

    }, [deletePhoto.status, addPhoto.status, props.processingPhoto])

    /**
     * Sets given image as selected photo to be maximized on the screen
     * @param 
     */
    const maximizePhoto = (i: number): void => {
        if (!props.editingEnabled) {
            props.setSelectedImage(i)
        }
        else {
            console.warn('[LandmarkDetails]: Editing is enabled, can\'t maximize image')
        }
    }

    return (
        <>
            {photoStatusMessage ?
            // Loading indicator or error message
            <View style={{justifyContent: "space-evenly", alignItems: "center"}}>
                <Text style={{color: 'white', fontSize: 20, marginBottom: 40}}>{photoStatusMessage}</Text>
                {
                    photoError ? 
                    <PrimaryButton text="Okay" style={{borderColor: 'white', borderWidth: 1}} onPress={() => props.toggleLmDetails(false)} />: 
                    <ActivityIndicator color='white' size="large"/>
                }
            </View> : 
            // Photo gallery
            <>
                <PhotoPicker 
                    multiple={true} 
                    menuType="alert" 
                    onReceivedPhotoResult={async result => await _onPhotoSelected(result)} 
                    photoSourceMenuOpened={photoSourceMenuOpened} 
                    onBeforeLaunchPicker={() => props.setProcessingPhoto(true)} 
                    cancel={() => {props.setProcessingPhoto(false) ; togglePhotoSourceMenu(false)}}/> 
                {props.landmark?.photos?.length > 0 ? 
                // landmark has photos, display them
                <>
                    <ScrollView nestedScrollEnabled={true} contentContainerStyle={{alignItems: 'center'}} style={{flexDirection: 'row', marginBottom: 5, alignSelf: 'center'}} horizontal={true}>
                        {props.landmark?.photos?.length < 5 && props.profileId == props.landmark?.user ? <IconButton style={{alignSelf: 'center', padding: 10, opacity: .5, marginLeft: 10}} color='white' size={30} icon="plus" onPress={() => togglePhotoSourceMenu(true)} />: null}
                        {props.landmark?.photos?.map((photo, i) => {    
                            
                            return (
                                <TouchableOpacity activeOpacity={1} key={i} style={{marginHorizontal: 1, padding: 14, zIndex: 11}} onPress={() => maximizePhoto(i)}>
                                    <Image style={{alignSelf: 'center', height: 300, width: 200}} source={{uri: 'data:image/png;base64,' + photo.image_b64}}/> 
                                    {props.landmark?.user == props.profileId ? <IconButton icon="times-circle" color="lightgray" style={{position: 'absolute', top: -2, right: 0}} size={25} onPress={() => props.tryDeletePhoto(photo.id)} /> : null}
                                </TouchableOpacity>
                            )
                        })}
                    </ScrollView>
                    
                </> : 
                // landmark has no photos, display add photo button
                <>
                {props.landmark?.user == props.profileId ?
                <TouchableOpacity style={{marginTop: 30, justifyContent: 'center', alignItems: 'center', opacity: .7}} onPress={() => {togglePhotoSourceMenu(true)}}>
                    <Text style={{fontSize: 20, marginBottom: 10, color: 'white'}}>Add photo</Text>
                    <FontAwesome name="plus" size={30} color='white' />
                </TouchableOpacity> : null}
                </>}
            </> }
        </>
    )
}