/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { useMutation, useQuery, useQueryClient } from "react-query";
import { makeAuthenticatedRequestAsync } from "../../libs/auth/core";
import { landmarkStore } from "../landmarks/store";
import { LMComment } from "../types";

export const useLandmarkComments = () => {
    const queryClient = useQueryClient();
    return useQuery<LMComment[]>(["getCommentsForLandmark", landmarkStore.selectedLandmarkId], async () => {
        if (landmarkStore.selectedLandmarkId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'GET',
                    url: `/api/comments/${landmarkStore.selectedLandmarkId}`
                }, "Error when retrieving comments");
                return response.data.reverse();
            } catch (error) {throw Error("Error when retrieving comments")}      
        }
    },
    {
        placeholderData: () => queryClient.getQueryData('getCommentsForLandmark'),
        staleTime: 1000,
        refetchInterval: 30000,
        refetchOnReconnect: true,
        refetchOnMount: false
    })
}

export const useAddComment = () => {
    const queryClient = useQueryClient();
    return useMutation(async (comment: LMComment) => {
        if (landmarkStore.selectedLandmarkId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    data: comment,
                    url: `/api/comments/`,
                }, "Error when retrieving comments");
                return response.data.reverse();
            } catch (error) {throw Error("Error when adding comment")}      
        }
    }, 
    {
        onSuccess: data => {
            queryClient.invalidateQueries('getCommentsForLandmark')
        }
    })
}

export const useUpdateComment = () => {
    const queryClient = useQueryClient()

    return useMutation(async (comment: LMComment) => {
        if (comment) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    data: comment,
                    url: `/api/comments/`,
                }, "Error when retrieving comments");
                return response.data.reverse();
            } catch (error) {throw new Error}      
        }
    }, {
        onSuccess: () => queryClient.invalidateQueries('getCommentsForLandmark'),  
        onError: () => queryClient.invalidateQueries('getCommentsForLandmark'),  
    })
}

export const useDeleteComment = () => {
    const queryClient = useQueryClient()
    return useMutation(async (commentId?: string | null) => {
        if (commentId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'DELETE',
                    url: `/api/comments/${commentId}`
                }, "Error when deleting comment");   
                return response.data;
            } catch (error) {throw new Error}   
        }
    }, {
        onSuccess: () => queryClient.invalidateQueries('getCommentsForLandmark'),  
        onError: () => queryClient.invalidateQueries('getCommentsForLandmark'),  
    })
}
