import { useMutation, useQuery, useQueryClient } from "react-query";
import { authStore } from "../../libs/auth/AuthStore";
import { makeAuthenticatedRequestAsync } from "../../libs/auth/core";
import { Landmark, LMPhoto } from "../types";
import { landmarkStore } from "./store";

const GET_LANDMARKS_QUERY_KEY = "getLandmarks"

export const useAllLandmarks = () => {
    const queryClient = useQueryClient()
    const getAllLandmarks = async () => {
        try {
            const response = await makeAuthenticatedRequestAsync({
                method: 'GET',
                url: '/api/landmarks/',
            }, "Something went wrong when retrieving landmarks");   
            return response.data
        } 
        catch {throw Error}
    }

    return useQuery<Landmark[], Error>(GET_LANDMARKS_QUERY_KEY, getAllLandmarks, 
    {
        placeholderData: () => queryClient.getQueryData('getLandmarks'),
        staleTime: 1000,
        refetchInterval: 30000,
        refetchOnReconnect: true,
        refetchOnMount: false
    })
}

export const useLandmarkDetails = () => {
    const queryClient = useQueryClient()
    const getLandmarkDetails = async () => {
        if (landmarkStore.selectedLandmarkId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'GET',
                    url: `/api/landmark/${landmarkStore.selectedLandmarkId}`,
                }, 'Error when retrieving landmark details.');   
                return response.data
            } catch (error) {throw Error}   
        }
    }

    return useQuery<Landmark, Error>(['getLandmark', landmarkStore.selectedLandmarkId], () => getLandmarkDetails(),
    {
        placeholderData: () => queryClient.getQueryData('getLandmark'),
        refetchOnReconnect: true,
        refetchOnMount: false
    })
}

export const useAddLandmark = () => {
    const queryClient = useQueryClient()
    return useMutation(async ({landmark, photos}: {landmark: Landmark, photos?: LMPhoto[]}) => {
        if (landmark) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    url: `/api/landmark/`,
                    data: {
                        landmark: landmark,
                        photos: photos
                    }
                }, 'Error when retrieving landmark details.');   
                return response.data
            } catch (error) {throw Error}   
        }
    }, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmarks')
        },  
    })
} 

export const useUpdateLandmark = () => {
    const queryClient = useQueryClient()
    return useMutation<Landmark, unknown, void, unknown>(async () => {
        if (landmarkStore.updatedLandmark) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'PUT',
                    data: landmarkStore.updatedLandmark,
                    url: `/api/landmark/`,
                }, "Error when updating landmark");   
                return response.data;
            } catch (error) {throw Error}

        }
        else {
            console.warn("[LandmarkData]: Can't update landmark. Given landmark value is null.")
        }
    },
    {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmarks')
        },  
    })
} 

export const useDeleteLandmark = () => {
    const queryClient = useQueryClient()
    return useMutation(async (landmarkId: string) => {
        if (landmarkId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'DELETE',
                    url: `/api/landmark/${landmarkId}`,
                }, 'Error when deleting landmark.');   
                return response.data
            } catch (error) {throw Error}   
        }
    }, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmarks')
        },  
    })
}

export const useAddLandmarkPhoto = () => {
    const queryClient = useQueryClient()
    return useMutation(async ({landmarkId, photo}: {landmarkId: string, photo: LMPhoto}) => {
        if (landmarkId && photo) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    url: `/api/landmark/${landmarkId}/photo/`,
                    data: {
                        photo: photo
                    }
                }, 'Error when adding photo to landmark.');   
                return response.data
            } catch (error) {throw Error}   
        }
    }, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmark')
        },  
    })
}

export const useDeleteLandmarkPhoto = () => {
    const queryClient = useQueryClient()
    return useMutation(async (photoId: string) => {
        if (photoId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'DELETE',
                    url: `/api/landmark/photos/${photoId}`,
                }, 'Error when deleting photo from landmark.');   
                return response.data
            } catch (error) {throw Error}   
        }
    }, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmark')
        },  
    })
}

export const useRateLandmark = () => {
    const queryClient = useQueryClient()
    return useMutation(async (rating: -1 | 1) => {
        if (landmarkStore.selectedLandmarkId && rating) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    data: {landmark_id: landmarkStore.selectedLandmarkId, rater_id: authStore.userId, rating: rating},
                    url: `/api/landmark/rate/`,
                }, 'Error when rating landmark.');   
                return response.data
            } catch (error) {
                throw Error
            }   
        }
    }, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmark')
        },  
    })
}

export const useCheckLandmarkRateUserPairing = () => {
    const checkIfLandmarkRatedByUser = async ({landmarkId, userId}: {landmarkId: string, userId: string}) => {
        if (landmarkId && userId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'GET',
                    url: `/api/landmark/check-rate-pairing/?landmark=${landmarkId}&user=${userId}`,
                }, "Error when checking if landmark is rated by user");   
                return response.data;
            } catch (error) {throw Error}      
        }
        else {
            console.warn("[LandmarkData]: Can't check landmark rating pairing. Given id values must not be null.")
        }
    }

    return useQuery<boolean>(
        ['checkIfRatedByUser', landmarkStore.selectedLandmarkId, authStore.userId], 
        () => checkIfLandmarkRatedByUser({
            landmarkId: landmarkStore.selectedLandmarkId, userId: authStore.userId
        }), 
        {
           refetchOnReconnect: true,
            refetchOnMount: false
        }
    )
}

