/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import axios, { AxiosRequestConfig } from "axios";
import { useMutation, useQuery, useQueryClient } from "react-query";
import { authStore } from "../../libs/auth/AuthStore";
import { makeAuthenticatedRequestAsync } from "../../libs/auth/core";
import { RegisterCredsValues, UserProfile } from "../types";

/**
 * Interface that contains data in a user's profile.
 */


export const useProfile = () => {
    const queryClient = useQueryClient();
    return useQuery<UserProfile>("getProfile", async () => {
        if (authStore.userId) {
            try {
                console.log('[Profile data]: Fetching profile...')
                const response = await makeAuthenticatedRequestAsync({
                    method: 'GET',
                    url: `/api/user-profile/${authStore.userId}/`,
                }, "Error fetching user profile");   
                console.log('[Profile data]: Profile retrieved')
                return response.data;
            } catch (error) {console.log('profile error'); throw new Error;}
        }
    }, {
        placeholderData: () => queryClient.getQueryData('getProfile'),
        refetchOnReconnect: true,
        refetchOnMount: true,               
    })
}

export const useUpdateProfile = () => {
    const queryClient = useQueryClient();

    return useMutation(async (values: RegisterCredsValues) => {
        if (authStore.userId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'PUT',
                    url: `/api/user-profile/${authStore.userId}/`,
                    data: values,
                }, "Error updating user profile");   
                return response.data
            } catch (error) {throw new Error;}    
        }
    }, {
        onSuccess: () => { queryClient.invalidateQueries('getProfile')},  
        onError: () => queryClient.invalidateQueries('getProfile'), 
    })
}

export const useChangePassword = () => {
    const queryClient = useQueryClient();

    return useMutation(async (password: string) => {
        if (authStore.userId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    url: `/api/user-profile/change-password/${authStore.userId}/`,
                    data: {password: password},
                }, "Error changing password");   
                return response.data;
            } catch (error) {throw new Error;}
        }   
    }, {
        onSuccess: () => { queryClient.invalidateQueries('getProfile')},  
        onError: () => queryClient.invalidateQueries('getProfile'),  
    })
}

export const useDeleteProfile = () => {
    const queryClient = useQueryClient();

    return useMutation(async () => {
        if (authStore.userId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'DELETE',
                    url: `/api/user-profile/${authStore.userId}/`,
                }, "Error when deleting user profile");   
                
                if (response.status == 200) {
                    await authStore.setAccessTokenAsync('')   
                    await authStore.setIdAsync('')   
                    await authStore.setNotificationTokenAsync('')   
                    await authStore.setRefreshTokenAsync('')   
                }
            } catch (error) {throw new Error;}
        }
    }, {
        onSuccess: () => { queryClient.invalidateQueries('getProfile')},  
        onError: () => queryClient.invalidateQueries('getProfile'),  
    })
}

export const useToggleTips = () => {
    const queryClient = useQueryClient();

    return useMutation(async () => {
        if (authStore.userId) {
            try {
                const response = await makeAuthenticatedRequestAsync({
                    method: 'POST',
                    url: `/api/user-profile/toggle-tips/${authStore.userId}/`,            
                }, "Error when toggling tips");   
                return response.data;
            } catch (error) {throw new Error;}
        
        }
    }, {
        onSuccess: () => { queryClient.invalidateQueries('getProfile')},  
        onError: () => queryClient.invalidateQueries('getProfile'),  
    })
}
