﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { observer } from 'mobx-react';
import React, { ReactElement, useEffect, useState } from 'react';
import { AppState, SafeAreaView } from 'react-native';
import { QueryClient } from 'react-query';
import { colors } from '../../utils/GlobalUtils';
import { authStore, reloadAuthStateFromStorage } from './AuthStore';

/**
 * Sub-root component of the app. Contains all global providers (NavigationContainer and SafeAreaProvider for React Navigation, QueryClientProvider for react-query) and is responsible for restricting unauthenticated users to the Intro screen by listening to {@link AuthStore}'s accessToken value.
 * @component
 */
const AuthStateProvider :  React.FC<{children(authenticated: boolean, checkingState: boolean): ReactElement}> = ({children}) => {
  /**
   * Flag that is switched on when the app is checking for tokens in the keystore and in memory. When true, "Logging you in.." and a spinner will be displayed to the user.
   */
  const [checkingState, setCheckingState] = useState<boolean>(true);
  const [authenticated, setAuthenticated] = useState<boolean>()

  /**
   * Checks if there is an access token available in {@link AuthStore}, then checks if that access token is valid by calling the API. 
   * If the response is valid, the access token will be stored in memory, otherwise the user will be directed to intro screen.
   */
  // const checkToken = async () => {
  //   // check both the mobx store and secure storage for the token
  //   console.log('[Authentication]: Checking for access token in memory...')
  //   let currentAccessToken = authStore.accessToken;
  //   if (!currentAccessToken) {
  //     console.log('[Authentication]: No access token in memory, checking in secure store...')
  //     currentAccessToken = await getItemAsync(SECURESTORE_ACCESSTOKEN);
  //   }

  //   if (!currentAccessToken) {
  //     console.log('[Authentication]: No access token in secure store, attempting to use a refresh token...')
  //     let refreshToken  = authStore.refreshToken;
  //     if (!refreshToken) {
  //       refreshToken = await getItemAsync(SECURESTORE_REFRESHTOKEN)
  //     }

  //     if (refreshToken) {
  //       await refreshAccessToken()
  //       currentAccessToken = authStore.accessToken
  //     }
  //   }  
    
  //   if (currentAccessToken) {
  //     console.log('[Authentication]: Found access token, testing its validity...')
  //     // check to see if the token is valid by making test call
  //     const requestConfig: AxiosRequestConfig = {
  //       method: 'GET',
  //       url: API_URL + "/api/me/",
  //       headers: { "Authorization": "Bearer " + currentAccessToken }
  //     };

  //     try {
  //       const response = await axios(requestConfig);
  //       if (response.status == 200) {
  //         await authStore.setAccessTokenAsync(currentAccessToken);
  //         await authStore.setRefreshTokenAsync(await getItemAsync(SECURESTORE_REFRESHTOKEN))
  //         await authStore.setIdAsync(response.data.id)
  //         console.log('[Authentication]: Access token valid.')
  //         setAuthenticated(true)
  //       }
  //     } catch (error) {
  //       // check if access token can be refreshed
  //       console.log(error)
  //       if (error.response.status == 401) {
  //         try {
  //           await refreshAccessToken();
  //           // update authorization header w/ new token
  //           await axios({...requestConfig, headers: { "Authorization": "Bearer " + authStore.accessToken }}); 
  //         } catch (error) {
            
  //         }
  //       }
  //       // something went wrong with the api call, log error and clear auth state
  //       reportAxiosError('[Authentication]: Something went wrong when retrieving an access token', error)
  //       await authStore.setAccessTokenAsync(null);
  //       await authStore.setRefreshTokenAsync(null);
  //       await authStore.setNotificationTokenAsync(null);
  //       await authStore.setIdAsync(null);
  //       setAuthenticated(false)
  //     }
  //   }
  //   else {
  //     // no access token was found, user will be taken to login
  //     console.log('[Authentication]: No access token was found, prompting user to login.')
  //     await authStore.setAccessTokenAsync(null);
  //     await authStore.setRefreshTokenAsync(null);
  //     await authStore.setNotificationTokenAsync(null);
  //     await authStore.setIdAsync(null);
  //     setAuthenticated(false)
  //   }

  //   setCheckingState(false);
  // }
  
  useEffect(() => {
    /**
     * useEffect hook that is responsible for registering an appState "change" handler that will call {@linkcode checkToken} each time the app is opened or closed on the device.
     * @memberOf Atlas
     */
    const checkAuthStateOnAppStateChange = async () => {
      if (AppState.currentState == 'active') {
        console.log('[Authentication]: App opened, checking auth state...')
        setCheckingState(true); 
        setAuthenticated(await authStore.authenticated)
        setCheckingState(false);
      }
    }
    checkAuthStateOnAppStateChange()
  }, [AppState.currentState]);

  useEffect(() => {
    /**
   * Calls {@linkcode checkToken} when a change to the access token stored in {@link AuthStore} is detected. 
   * @memberOf Atlas
   */
    const checkTokenOnAccessTokenChange = async () => {
      console.log('[Authentication]: Change to accessToken detected, checking token state...')
      setCheckingState(true);
      setAuthenticated(await authStore.authenticated)
      setCheckingState(false);
    }
    checkTokenOnAccessTokenChange()
  }, [authStore.accessToken]);

  return (
    <SafeAreaView style={{height: '100%', backgroundColor: colors.red}}>
      {children(authenticated, checkingState)}
    </SafeAreaView>
  );
}

export default observer(AuthStateProvider);

