﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import * as SecureStore from 'expo-secure-store';
import { getItemAsync } from 'expo-secure-store';
import { action, computed, makeObservable, observable } from "mobx";
import Config from 'react-native-config';
import { makeAuthenticatedRequestAsync } from './core';
import { AuthState } from './types';

export const SECURESTORE_ACCESSTOKEN = 'access'
export const SECURESTORE_IDTOKEN = 'userId'
export const SECURESTORE_NOTIFTOKEN = 'notif'
export const SECURESTORE_REFRESHTOKEN = 'refresh'

/**
 * A mobx store responsible for holding the current access token, refresh token and userId in memory and in Expo's [SecureStore]{@link https://docs.expo.dev/versions/latest/sdk/securestore/}.
 * Used by {@link Atlas}
 * @category Stores
 */
class AuthStore {
    /**
     * A string representing the OAuth2 access token issued to the user when they authenticate. It is sent in the Authorization header of all subsequent XHR requests in order to use authorized API endpoints.
     * For more information on OAuth2 and OpenIdConnect, visit {@link https://openid.net/connect/} and {@link https://oauth.net/2/}.
     * @memberOf AuthStore
     */
    accessToken?: string | null = null;
    /**
     * A string representing the OAuth2 refresh token issued to the user when they authenticate. It is sent to the identity provider to retrieve a new access token when the previous access token expires.
     * @memberOf AuthStore
     */
    refreshToken?: string | null = null;
    /**
     * A string representing the OAuth2 refresh token issued to the user when they authenticate. It is used to retrieve the users information from the API and to link objects created by the user to the user.
     */
    userId?: string = undefined;
    /**
     * A string representing the expo notification token.
     */
    notificationToken?: string = undefined;
    /**
     * Whether authentication state is currently being checked
     */
    checkingAuth: boolean = false;

    /**
     * Constructor. Uses mobx's [makeObservable]{@linkcode https://mobx.js.org/observable-state.html} to register actions and observable elements.
     */
    constructor() {
        makeObservable(this, {
            accessToken: observable,
            refreshToken: observable,
            userId: observable,
            notificationToken: observable,
            authenticated: computed,
            checkingAuth: observable,
            setAccessTokenAsync: action,
            setRefreshTokenAsync: action,
            setIdAsync: action,
            setNotificationTokenAsync: action
        });
    }

    get authenticated() : Promise<boolean> {
        const checkIfAuthenticated = async () => {
            const validateAccessToken = async (token: string): Promise<boolean> => {
                try {
                    console.log('[Authentication]: Access token found, checking if it is valid...');
                    const response = await makeAuthenticatedRequestAsync({
                        method: 'GET',
                        url: Config.API_URL + "/api/me/",
                        headers: { "Authorization": "Bearer " + token }
                    }, "Something went wrong when trying to validate the current access token")   
                    if (response.status === 200) {
                        console.log('[Authentication]: Access token is valid...');
                        await reloadAuthStateFromStorage()
                        return true
                    }
                    else {
                        await clearAuthState()
                        console.log('[Authentication]: Access token is invalid, prompting the user to login...');
                        return false
                    }
                } catch (error) {
                    await clearAuthState()
                    console.log('[Authentication]: Access token is invalid, prompting the user to login...');
                    return false
                }
            }
            
            if (this.accessToken) {
                console.log('[Authentication]: Access token found in memory, checking if it is valid...');
                return validateAccessToken(this.accessToken)
            }
            else {
                let token = await getItemAsync(SECURESTORE_ACCESSTOKEN)
                if (token) {
                    console.log('[Authentication]: Access token found on disk, checking if it is valid...');
                    await authStore.setAccessTokenAsync(token)
                    return validateAccessToken(token)
                }
                else {
                    console.log('[Authentication]: No access token found, prompting the user to login...');
                    return false
                }
            }
        }

        return checkIfAuthenticated()
    }

    /**
     * Uses the given access token value to set {@linkcode accessToken} and the 'access' element in the secure store.
     */
    async setAccessTokenAsync(tokenValue: string | null) {
        this.accessToken = tokenValue;
        if (this.accessToken) {
            await SecureStore.setItemAsync(SECURESTORE_ACCESSTOKEN, this.accessToken);    
        }
        else 
        {
            await SecureStore.deleteItemAsync(SECURESTORE_ACCESSTOKEN);    
        }
    }

    /**
     * Uses the given refresh token value to set {@linkcode refreshToken} and the 'refresh' element in the secure store.
     */
    async setRefreshTokenAsync(tokenValue?: string) {
        this.refreshToken = tokenValue;
        if (this.refreshToken) {
            await SecureStore.setItemAsync(SECURESTORE_REFRESHTOKEN, this.accessToken);    
        }
        else 
        {
            await SecureStore.deleteItemAsync(SECURESTORE_REFRESHTOKEN);    
        }
    }

    /**
     * Uses the given user id value to set {@linkcode userId} and the 'id' element in the secure store.
     */
    async setIdAsync(id?: string) {
        this.userId = id;
        if (this.userId) {
            await SecureStore.setItemAsync(SECURESTORE_IDTOKEN, this.userId);    
        }
        else 
        {
            await SecureStore.deleteItemAsync(SECURESTORE_IDTOKEN);    
        }
    }

    /**
     * Uses the given notification token value to set {@linkcode notificationToken} and the 'notif-token' element in the secure store.
     */
     async setNotificationTokenAsync(token?: string) {
        this.notificationToken = token;
        if (this.notificationToken) {
            await SecureStore.setItemAsync(SECURESTORE_NOTIFTOKEN, this.notificationToken);    
        }
        else 
        {
            await SecureStore.deleteItemAsync(SECURESTORE_NOTIFTOKEN);    
        }
    }
}

export const authStore = new AuthStore();

export const reloadAuthStateFromStorage = async () => {
    try {
        await authStore.setAccessTokenAsync(await SecureStore.getItemAsync(SECURESTORE_ACCESSTOKEN))
        await authStore.setRefreshTokenAsync(await SecureStore.getItemAsync(SECURESTORE_REFRESHTOKEN))
        await authStore.setIdAsync(await SecureStore.getItemAsync(SECURESTORE_IDTOKEN))
    } catch (error) {
        console.error("Something went wrong when reloading auth state: " + error)   
    }
}

export const setAuthState = async (state: AuthState) => {
    try {
        await authStore.setAccessTokenAsync(state.accessToken)
        await authStore.setRefreshTokenAsync(state.refreshToken)
        await authStore.setIdAsync(state.userId)      
    } catch (error) {
        console.error("Something went wrong when setting auth state: " + error)   
    }
}

export const clearAuthState = async () => {
    try {
        await authStore.setAccessTokenAsync("")
        await authStore.setRefreshTokenAsync("")
        await authStore.setIdAsync("")      
    } catch (error) {
        console.error("Something went wrong when clearing auth state: " + error)   
    }
}