
/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import axios, { AxiosError, AxiosRequestConfig } from "axios"
import { AuthRequestConfig, DiscoveryDocument, loadAsync } from "expo-auth-session"
import jwtDecode from "jwt-decode"
import Config from "react-native-config"
import { authStore } from "./AuthStore"
import { AuthenticationResult, AuthState, IdToken, } from "./types"

const setAuthState = async (state: AuthState) => {
    try {
        await authStore.setAccessTokenAsync(state.accessToken)
        await authStore.setRefreshTokenAsync(state.refreshToken)
        await authStore.setIdAsync(state.userId)      
    } catch (error) {
        console.error("Something went wrong when setting auth state: " + error)   
    }
    
}

const clearAuthState = async () => {
    try {
        await authStore.setAccessTokenAsync(null)
        await authStore.setRefreshTokenAsync(null)
        await authStore.setIdAsync(null)        
    } catch (error) {
        console.error("Something went wrong when clearing auth state: " + error)   
    }
}

export const cnpAxios = axios.create({
    baseURL: Config.API_URL,
  });
  

export const makeUnauthenticatedRequestAsync = async (config: AxiosRequestConfig, errorMessage: string) => {
    try {
        const response = await cnpAxios(config)
        return response   
    } catch (error) {  
        reportAxiosError(errorMessage, error, true)
    }
} 

/**
 * If there is a refresh token available, attempts to use it to obtain a new, valid access token.
 * Used in {@link Atlas}
 * @memberOf useAuth
 */
export const refreshAccessToken = async () => {
    if (authStore.refreshToken) {
        try {
            const response = await makeUnauthenticatedRequestAsync({
                method: 'POST',
                url: '/o/token/',
                data: {grant_type: 'refresh_token', refresh_token: authStore.refreshToken, client_id: 'atlas.mobile'},
                headers: { 'Content-Type': "application/x-www-form-urlencoded" }
            }, "Something went wrong when refreshing the current access token") 
            
            const idToken = jwtDecode(response.data.id_token) as IdToken;
            await setAuthState({
                accessToken: response.data.access_token,
                refreshToken: response.data.refresh_token,
                userId: idToken.sub
            })   

            console.info('Successfully refreshed access token, re-attempting initial request...')
        } catch (error) {
            //await clearAuthState()
        }
    }
}

export const registerAsync = async (registerData: FormData): Promise<AuthenticationResult> => {
    console.log("[Registration]: Sending registration request...")
    try {
        const response = await makeUnauthenticatedRequestAsync({
            method: 'post',
            url: '/api/register/',
            timeout: 50000,
            data: registerData,
        }, "Error when trying to register user");
        return {success: true, message: 'Success! Congratulations on registering for your new account, you can now login!', payload: null} 
      
    } catch (error) {
        return {success: true, message: 'Something went wrong when trying to set up your account. Please try again.', payload: null}   
    }
}

/**
     * Function that initiates the login flow. It opens up with the in app browser and sends an authorization request to the API, which redirects the user to the backend login page. 
     * If the credentials entered are valid, the OAuth Authorization Code flow will occur, which results in the user recieving an access token and refresh token which are then stored in {@link AuthStore}.
     * */
 export const login = async (requestConfig: AuthRequestConfig, discovery: DiscoveryDocument): Promise<AuthenticationResult> => {
    console.log('[Authentication]: User is attempting to login, opening web portal login...')

    // initiate authentication request to the server 
    const request = await loadAsync(requestConfig, discovery)  

    // handle authentication response from the server
    const response = await request.promptAsync(discovery);

    // if succesful, prepare a request for an access/id token
    if (response.type == "success" && request.codeVerifier) {
        console.log('[Authentication]: User authentication was successful.')
        const tokenData = new URLSearchParams();
        tokenData.append('grant_type', 'authorization_code');
        tokenData.append('client_id', 'atlas.mobile');
        tokenData.append('code', response.params.code);
        tokenData.append('redirect_uri', request.redirectUri);
        tokenData.append('code_verifier', request.codeVerifier);  
        console.log('[Authentication]: Attempting to retrieve access token...')

        // send the token request
        try {

            const response = await makeUnauthenticatedRequestAsync({
                url: '/o/token/',
                method: 'POST',
                data: tokenData,
                headers: {'Content-Type': "application/x-www-form-urlencoded"}
            }, "Something went wrong when tryiong to exchange auth code for access token")

            const tokenResponse = response.data;

            // if its a successful response, decode the jwt id token, and store the tokens in the corresponding stores
            const idToken = jwtDecode(tokenResponse.id_token) as IdToken;

            await authStore.setAccessTokenAsync(tokenResponse.access_token);
            await authStore.setRefreshTokenAsync(tokenResponse.refresh_token);
            await authStore.setIdAsync(idToken.sub)

            console.log('[Authentication]: Tokens successfully retrieved.')

            return {success: true}
        } catch (error) {
            return {success: false}
        } 
    }
}   

export const logout = async () => {
    console.log('[Authentication]: User is attempting to logout...')
    const tokenParams = new URLSearchParams();
    tokenParams.append('client_id', 'atlas.mobile');
    tokenParams.append('token', authStore.accessToken as string);
    
    try {
        const response = await makeUnauthenticatedRequestAsync(({
            method: 'POST',
            url: '/o/revoke-token/',
            data: tokenParams,
            headers: {'Content-Type': 'application/x-www-form-urlencoded'}
        }), 'Something went wrong when logging out user')   
        if (response.status == 200) {
            await clearAuthState()   
        }
    } catch (error) {
        console.log('[Authentication]: Something went wrong when logging out user')
    }
}

export const makeAuthenticatedRequestAsync = async (config: AxiosRequestConfig, errorMessage: string) => {
    try {
        const response = await cnpAxios({...config, headers: {...config.headers,  "Authorization": "Bearer " + authStore.accessToken }})
        return response   
    } catch (error) {
        if (error.response.status === 401 || error.response.status === 403) {
            try {
              await refreshAccessToken();
              // update authorization header w/ new token
              await cnpAxios({...config, headers: { "Authorization": "Bearer " + authStore.accessToken }}); 
            } catch (error) {
              reportAxiosError(errorMessage, error)
            }
        }
        else {
            reportAxiosError(errorMessage, error)
        }
    }
} 

const reportAxiosError = (desc: string,  error: AxiosError, printResponse?: boolean) => {
  let errorString = `XHR error: ${desc}\nError code: ${error.response?.status}\nError message: ${error.message}`;

  if (printResponse) {
      errorString + "\nError response: " + error.response;
  }

  console.error(errorString);
}