import { FontAwesome } from "@expo/vector-icons"
import { useNavigation, useRoute } from "@react-navigation/native"
import { createStackNavigator, StackNavigationProp } from "@react-navigation/stack"
import { observer } from "mobx-react"
import React, { useEffect } from "react"
import { Image, ScrollView, View } from "react-native"
import { Chip } from "react-native-paper"
import { IconButton } from "../components/Buttons"
import IndoorMap from "../components/Map/MainMapComponent/IndoorMap"
import mapStyles from "../components/Map/MainMapComponent/Map.styles"
import OutdoorMap, { AuthTabsMapRouteProp } from "../components/Map/MainMapComponent/OutdoorMap"
import { useMapState } from "../components/Map/MainMapComponent/useMapState"
import AddLandmarkPanel from "../components/Map/Panels/AddLandmarkPanel"
import { FilterPanel } from "../components/Map/Panels/FilterPanel/FilterPanel"
import LandmarkDetails from "../components/Map/Panels/LandmarkDetailsPanel/LandmarkDetails"
import { useAllLandmarks } from "../data/landmarks/hooks"
import { landmarkStore } from "../data/landmarks/store"
import { Landmark } from "../data/types"
import { authStore } from "../libs/auth/AuthStore"
import { colors, lmTypes } from "../utils/GlobalUtils"
import { AuthTabsNavigationProp } from "./AuthorizedNavigator"

const MapStackNavigator = createStackNavigator()

export type MapStackParamList = {
    Outdoor: {selectedLandmark: string, selectedLandmarks: string[]},
    Indoor: React.FC,
}

export type MapStackNavigationProp = StackNavigationProp<MapStackParamList>

interface MapNavigatorProps {
    authNavigation: AuthTabsNavigationProp
    route: AuthTabsMapRouteProp
}

const tabBarOptions = {
    //keyboardHidesTabBar: true,
    activeTintColor: 'white',
    activeBackgroundColor: '#e35555',
    inactiveTintColor: "lightgray",
    
    style: {backgroundColor: colors.red, height: 60, justifyContent: 'center'},
    labelStyle: {marginBottom: 7},
    iconStyle: {marginBottom: 7}
} 


const MapNavigator: React.FC<MapNavigatorProps> = ({route}) => {
    const mapState = useMapState()
    const authNavigation = useNavigation() as AuthTabsNavigationProp
    const authRoute = useRoute() as AuthTabsMapRouteProp

    const allLandmarksQuery = useAllLandmarks()

    /**
     * Clear selected landmark when landmark details panel is closed
     */
     useEffect(() => {
        if (!mapState.lmDetailsVisible) {
            landmarkStore.setSelectedLandmarkId('')
        }
    }, [mapState.lmDetailsVisible])

    useEffect(() => {
        const refetchLandmarksOnFilterOptionsChange = async () => {
            applyFilters(allLandmarksQuery.data)
            console.log('[Map]: Filters changed')
        }
        refetchLandmarksOnFilterOptionsChange()
    }, [mapState.lmFilteredTypes, mapState.onlyOwned, mapState.minLmRating])

    const applyFilters = (landmarks: Landmark[]): Landmark[] => {
        if (landmarks?.length > 0) {
            if (mapState.minLmRating > 0) {
                landmarks = landmarks?.filter(lm => lm.rating >= mapState.minLmRating);
            }
        
            if (mapState.lmFilteredTypes?.length > 0) {
                if (mapState.lmFilteredTypes.length > 0) {
                    landmarks = landmarks?.filter(lm => mapState.lmFilteredTypes.includes(lm.landmark_type));   
                }
            }
        
            if (mapState.onlyOwned) {
                landmarks = landmarks?.filter(lm => lm.user == authStore.userId);
            }
        }
    
        return landmarks
    }

    /**
     * Triggered by on a {@link Landmark} displayed on the map. 
     * Sets {@linkcode selectedLandmark} to the pressed {@link Landmark} object's value and toggles the {@link LandmarkDetails} modal.
     */
    const focusLandmark = (landmark: Landmark) => {
        if (landmarkStore.selectedLandmarkId == landmark.id) {
            mapState.toggleLmDetails(true)
        }
        else {
            landmarkStore.setSelectedLandmarkId(landmark.id);
        }
    }

        /**
     * Triggered by long pressing on the map. 
     * Sets {@linkcode newLandmarkState} to a skeleton {@link Landmark} object that only contains the pressed coordinates.
     * Triggers {@link openAddLandmark} via useEffect because the asyncronous nature of useState does not set the coordinates fast enough to toggle the
     * modal directly through this method (this issue shows up in many other parts of the app where a modal is toggled by a boolean, and is solved in the same way).
     */
    const promptAddLandmark = (longitude?: number, latitude?: number) => {
        console.log('[Map]: Opening add landmark panel...')
        landmarkStore.setNewLandmark({latitude: latitude, longitude: longitude});
        mapState.toggleLmAdd(true)
        mapState.toggleLmDetails(false)   
    }

    return (
        <View style={{flex: 1}}>
            <MapStackNavigator.Navigator screenOptions={{ headerShown: false }} initialRouteName="Outdoor">
                <MapStackNavigator.Screen name="Outdoor" >
                    {({navigation}) => 
                    <OutdoorMap 
                        promptAddLandmark={promptAddLandmark}
                        toggleLmAdd={mapState.toggleLmAdd}
                        toggleLmDetails={mapState.toggleLmDetails}
                        applyFilters={applyFilters}
                        focusLandmark={focusLandmark} 
                        route={authRoute} 
                        mapNavigation={navigation} 
                        authNavigation={authNavigation}
                        landmarks={allLandmarksQuery.data ? applyFilters(allLandmarksQuery.data) : []}/>}
                </MapStackNavigator.Screen>
                <MapStackNavigator.Screen name="Indoor" component={IndoorMap} />
            </MapStackNavigator.Navigator>
            {/*Filter chips and button*/}
            {!mapState.filterVisible ? 
            <View style={{top: 10, marginLeft: 40, marginRight: 20, position: 'absolute', flexDirection: "row-reverse", justifyContent: 'flex-end'}}>
                <IconButton size={20} color={colors.red} style={[mapStyles.filterButton]} icon="filter" onPress={() => mapState.toggleFilter(true)}/>
                <ScrollView horizontal={true} contentContainerStyle={{alignItems: 'center'}} style={{marginHorizontal: 10, flexDirection: 'row'}}>
                {mapState.onlyOwned ? <Chip avatar={(<FontAwesome name="user" size={20} color='gray' style={{textAlign: 'center', textAlignVertical: 'center'}} />)} style={{borderWidth: 1, borderColor: 'lightgray', marginRight: 5, marginLeft: 10}} onClose={() => mapState.toggleOnlyOwned(false)}>My landmarks</Chip> : null}
                {mapState.lmFilteredTypes.map((type, i) => {
                    return (
                        <Chip key={i} avatar={(<Image style={{height: 22, width: 17}} source={lmTypes[type].image} />)} style={{borderWidth: 1, borderColor: 'lightgray', marginHorizontal: 5, justifyContent: 'center'}} onClose={() => mapState.setLmTypeFilter(mapState.lmFilteredTypes.filter(t => t !== type))}>{lmTypes[type].label}</Chip>
                    )
                })}
                {mapState.minLmRating > 0 ? <Chip avatar={(<FontAwesome name="star" size={20} color='gray' style={{textAlign: 'center', textAlignVertical: 'center'}} />)} style={{borderWidth: 1, borderColor: 'lightgray', marginLeft: 5, marginRight: 10}} onClose={() => mapState.setMinLmRating(0)}>Minimum rating: {mapState.minLmRating}</Chip> : null}
                </ScrollView>
            </View> : null}
            <AddLandmarkPanel 
                setNewLandmark={landmarkStore.setNewLandmark} 
                setVisible={mapState.toggleLmAdd} 
                newLandmark={landmarkStore.newLandmark}
                visible={mapState.lmAddVisible} />
            <LandmarkDetails 
                visible={mapState.lmDetailsVisible}
                toggleLmDetails={mapState.toggleLmDetails}
                setLandmark={landmarkStore.setSelectedLandmarkId} 
                toggleDetailsPanel={mapState.toggleLmDetails} 
                setEditing={mapState.toggleLmDetailsEditing} 
                editingEnabled={mapState.lmDetailsEditing} 
                landmarkId={landmarkStore.selectedLandmarkId} />
            <FilterPanel 
                visible={mapState.filterVisible}
                toggleOnlyOwned={mapState.toggleOnlyOwned} 
                onlyOwned={mapState.onlyOwned} toggleFilter={mapState.toggleFilter} 
                setMinLmRating={mapState.setMinLmRating} 
                setLmFilteredTypes={mapState.setLmTypeFilter} 
                lmFilteredTypes={mapState.lmFilteredTypes} 
                minLmRating={mapState.minLmRating} />
        </View>
    )
}

export default observer(MapNavigator)